#region [ License information          ]
/* ************************************************************
 *
 * Copyright (c) Bjrn Bouet Smith, 2010, 2011
 *
 * This source code is subject to terms and conditions of 
 * Microsoft Public License (Ms-PL).
 * 
 * A copy of the license can be found in the license.txt
 * file at the root of this distribution. If you can not 
 * locate the License, please send an email to bjornsmith@gmail.com
 * 
 * By using this source code in any fashion, you are 
 * agreeing to be bound by the terms of the Microsoft 
 * Permissive License.
 *
 * You must not remove this notice, or any other, from this
 * software.
 *
 * ************************************************************/
#endregion

using System;
using Smith.AMC.Operations;

namespace Smith.AMC.Network
{
    /// <summary>
    /// Interface for class that can write commands to a memcached server.
    /// </summary>
    public interface IClientSocket : IDisposable
    {
        /// <summary>
        /// Gets or sets the owner ID of this socket.
        /// This ID is the ID of the memcached server
        /// </summary>
        int OwnerID
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="IClientSocket"/> is dead.
        /// </summary>
        bool Dead
        {
            get;
            set;
        }

        /// <summary>
        /// Begins an asynchronous write operation against the memcached server.
        /// </summary>
        /// <typeparam name="T">The type of data to add.</typeparam>
        /// <param name="operation">The operation to write to the socket.</param>
        /// <param name="asyncCallBack">The <see cref="System.AsyncCallback"/> delegate that is executed when the BeginAdd operation completes</param>
        /// <param name="state">An object that contains any additional user-defined data.</param>
        /// <returns>
        /// An <see cref="System.IAsyncResult"/> that represents the asynchronous call.
        /// </returns>
        IAsyncResult BeginWrite<T>(Operation operation, AsyncCallback asyncCallBack, object state);

        /// <summary>
        /// Handles the end of the asynchronous write operation
        /// </summary>
        /// <typeparam name="T">Type of item added</typeparam>
        /// <param name="result">An <see cref="System.IAsyncResult"/> that represents the asynchronous call.</param>
        /// <returns>
        /// true if the add operation was a success; false otherwise
        /// </returns>
        OperationResponse<T> EndWrite<T>(IAsyncResult result);
    }
}