﻿#region [ License information          ]
/* ************************************************************
 *
 * Copyright (c) Bjørn Bouet Smith, 2010, 2011
 *
 * This source code is subject to terms and conditions of 
 * Microsoft Public License (Ms-PL).
 * 
 * A copy of the license can be found in the license.txt
 * file at the root of this distribution. If you can not 
 * locate the License, please send an email to bjornsmith@gmail.com
 * 
 * By using this source code in any fashion, you are 
 * agreeing to be bound by the terms of the Microsoft 
 * Public License.
 *
 * You must not remove this notice, or any other, from this
 * software.
 *
 * ************************************************************/
#endregion

namespace Smith.AMC.Operations
{
    /// <summary>
    /// Represents an operation that will increment a ulong value stored in the memcached server.
    /// </summary>
    public class IncrementOperation : Operation
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="IncrementOperation"/> class.
        /// </summary>
        public IncrementOperation()
            : base(OperationNames.Increment)
        {
            IsOneLine = true;
        }

        /// <summary>
        /// Gets or sets the data, overridden to be a ulong
        /// </summary>
        public new ulong Data
        {
            get;
            set;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return string.Format("{0} {1} {2}\r\n", Name, Key, Data);
        }
    }
}
